@@ -122,7 +122,7 @@ class DFFormat(object):
         return ("DFFormat(%s,%s,%s,%s)" %
                 (self.type, self.name, self.format, self.columns))
 
-
+# Swiped into mavgen_python.py
 def to_string(s):
     '''desperate attempt to convert a string regardless of what garbage we get'''
     try:
@@ -194,6 +194,16 @@ class DFMessage(object):
             v *= self.fmt.msg_mults[i]
         return v
 
+    def __setattr__(self, field, value):
+        '''override field setter'''
+        if not field[0].isupper() or not field in self.fmt.colhash:
+            super(DFMessage,self).__setattr__(field, value)
+        else:
+            i = self.fmt.colhash[field]
+            if self.fmt.msg_mults[i] is not None and self._apply_multiplier:
+                value /= self.fmt.msg_mults[i]
+            self._elements[i] = value
+
     def get_type(self):
         return self.fmt.name
 
@@ -450,10 +460,16 @@ class DFReader(object):
         self.verbose = False
         self.params = {}
         self._flightmodes = None
+        self.messages = {}
 
     def _rewind(self):
         '''reset state on rewind'''
-        self.messages = {'MAV': self}
+        # be careful not to replace self.messages with a new hash;
+        # some people have taken a reference to self.messages and we
+        # need their messages to disappear to.  If they want their own
+        # copy they can copy.copy it!
+        self.messages.clear()
+        self.messages['MAV'] = self
         if self._flightmodes is not None and len(self._flightmodes) > 0:
             self.flightmode = self._flightmodes[0][0]
         else:
@@ -743,7 +759,8 @@ class DFReader_binary(DFReader):
             hdr = self.data_map[ofs:ofs+3]
             if hdr[0] != HEAD1 or hdr[1] != HEAD2:
                 print("bad header 0x%02x 0x%02x" % (u_ord(hdr[0]), u_ord(hdr[1])), file=sys.stderr)
-                break
+                ofs += 1
+                continue
             mtype = u_ord(hdr[2])
             self.offsets[mtype].append(ofs)
 
@@ -756,6 +773,8 @@ class DFReader_binary(DFReader):
                 self._parse_next()
                 fmt = self.formats[mtype]
                 lengths[mtype] = fmt.len
+            elif self.formats[mtype].instance_field is not None:
+                self._parse_next()
 
             self.counts[mtype] += 1
             mlen = lengths[mtype]
