@@ -1460,6 +1460,8 @@ class mavmmaplog(mavlogfile):
                 mtype = u_ord(self.data_map[ofs+13])
                 mlen += 8
             elif marker == MARKER_V2:
+                if ofs+8+10 > self.data_len:
+                    break
                 mtype = u_ord(self.data_map[ofs+15]) | (u_ord(self.data_map[ofs+16])<<8) | (u_ord(self.data_map[ofs+17])<<16)
                 mlen += 12
                 incompat_flags = u_ord(self.data_map[ofs+10])
@@ -1867,7 +1869,8 @@ mode_mapping_apm = {
     10 : 'AUTO',
     11 : 'RTL',
     12 : 'LOITER',
-    14 : 'LAND',
+    13 : 'TAKEOFF',
+    14 : 'AVOID_ADSB',
     15 : 'GUIDED',
     16 : 'INITIALISING',
     17 : 'QSTABILIZE',
@@ -1876,6 +1879,7 @@ mode_mapping_apm = {
     20 : 'QLAND',
     21 : 'QRTL',
     22 : 'QAUTOTUNE',
+    23 : 'QACRO',
     }
 mode_mapping_acm = {
     0 : 'STABILIZE',
@@ -1923,6 +1927,7 @@ mode_mapping_tracker = {
     0 : 'MANUAL',
     1 : 'STOP',
     2 : 'SCAN',
+    4 : 'GUIDED',
     10 : 'AUTO',
     16 : 'INITIALISING'
     }
@@ -2329,41 +2334,48 @@ def dump_message_verbose(f, m):
         try:
             units = m.fieldunits_by_name[fieldname]
             # perform simple unit conversions:
+            divisor = None
             if units == "d%":
-                value = value / 10.0
+                divisor = 10.0
                 units = "%"
             if units == "c%":
-                value = value / 100.0
+                divisor = 100.0
                 units = "%"
 
             if units == "cA":
-                value = value / 100.0
+                divisor = 100.0
                 units = "A"
 
             elif units == "cdegC":
-                value = value / 100.0
+                divisor = 100.0
                 units = "degC"
 
             elif units == "cdeg":
-                value = value / 100.0
+                divisor = 100.0
                 units = "deg"
 
             elif units == "degE7":
-                value = value / 10000000.0
+                divisor = 10000000.0
                 units = "deg"
 
             elif units == "mG":
-                value = value / 1000.0
+                divisor = 1000.0
                 units = "G"
 
             elif units == "mrad/s":
-                value = value / 1000.0
+                divisor = 1000.0
                 units = "rad/s"
 
             elif units == "mV":
-                value = value / 1000.0
+                divisor = 1000.0
                 units = "V"
 
+            if divisor is not None:
+                if type(value) == list:
+                    value = [x/divisor for x in value]
+                else:
+                    value = value / divisor
+
             # and give radians in degrees too:
             if units == "rad":
                 value = "%s%s (%sdeg)" % (value, units, math.degrees(value))
